"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extRdflib = _interopRequireDefault(require("ext-rdflib"));

var _nodeFetch = _interopRequireWildcard(require("node-fetch"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _construct(Parent, args, Class) { if (isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

/**
 * Node Status:
 * 
 * Locally undetermined: There are multiple local nodes satisfying the criteria specified for this object 
 * Blank: The object represents a blank node in a graph that is locally available
 * Unresolved: This node is identified by a URI that has not yet been derefernced
 * 
 * @type type
 */
var Headers = function (h) {
  return h ? h : window.Headers;
}(_nodeFetch["default"].Headers);

function GraphNode() {
  return _construct(GraphNode.Impl, Array.prototype.slice.call(arguments));
}

GraphNode.Impl =
/*#__PURE__*/
function () {
  function _class(nodes, graph, sources) {
    _classCallCheck(this, _class);

    this._graph = graph;

    if (Array.isArray(nodes)) {
      this.nodes = nodes;
      /*if (this.nodes.length === 0) {
          throw "Can't represent empty set of nodes";
      }*/
    } else {
      this.nodes = [nodes];
    }

    this.sources = sources;
  }

  _createClass(_class, [{
    key: "fetch",
    value: function fetch() {
      var _this = this;

      if (this.termType !== "NamedNode" || this.sources && this.sources.indexOf(this.value.split("#")[0]) > -1) {
        return Promise.resolve(this);
      } else {
        //TODO extend existing graph?
        var uri = this.value.split("#")[0];
        return GraphNode.rdfFetch(uri).then(function (response) {
          return response.graph();
        }).then(function (graph) {
          return GraphNode(_this.node, graph, [uri]);
        });
      }
    }
    /*
     * 
     * @param {type} f
     * @returns {unresolved} a promise that is satisfied when all promises returned by f are resolved
     */

  }, {
    key: "each",
    value: function each(f) {
      var _this2 = this;

      var results = this.nodes.map(function (node) {
        return f(GraphNode([node], _this2.graph, _this2.sources));
      });
      return Promise.all(results);
    }
  }, {
    key: "fetchEach",
    value: function fetchEach(f) {
      var _this3 = this;

      var results = this.nodes.map(function (node) {
        return GraphNode([node], _this3.graph, _this3.sources).fetch().then(f);
      });
      return Promise.all(results);
    }
    /**
     * Returns a GraphNode for each node represented by this GraphNode
     */

  }, {
    key: "split",
    value: function split() {
      var _this4 = this;

      return this.nodes.map(function (node) {
        return GraphNode([node], _this4.graph, _this4.sources);
      });
    }
  }, {
    key: "out",
    value: function out(predicate) {
      var nodes = this.graph.each(this.node, predicate);
      /*if (nodes.length === 0) {
          throw "No property "+predicate+" on "+this.node;
      }*/

      return GraphNode(nodes, this.graph, this.sources);
    }
  }, {
    key: "in",
    value: function _in(predicate) {
      var statements = this.graph.statementsMatching(undefined, predicate, this.node);
      /*if (statements.length === 0) {
          throw "No property "+predicate+" pointing to "+this.node;
      }*/

      return GraphNode(statements.map(function (statement) {
        return statement.subject;
      }), this.graph, this.sources);
    }
  }, {
    key: "graph",
    get: function get() {
      if (!this._graph) {
        throw Error("Operation not possible as no Graph is available, try fetching first");
      }

      return this._graph;
    }
  }, {
    key: "node",
    get: function get() {
      if (this.nodes.length !== 1) {
        throw Error("Operation not possible as this GraphNode is underdetermined");
      }

      return this.nodes[0];
    }
  }, {
    key: "termType",
    get: function get() {
      return this.node.termType;
    }
  }, {
    key: "value",
    get: function get() {
      ;
      return this.node.value;
    }
  }]);

  return _class;
}();
/**
 * 
 * Fetches an RDF graph. If the server return 401 the login process will be 
 * started upon which the fetch will be retried.
 *
 * @param uri {string} The URI to be fetched
 * @param options passed to $rdf.Fetcher
 * @param login {boolean} The login function to be called, optional
 *
 * @return {Promise<Response>} Response has a `graph`property with the rertived graph
 */


GraphNode.rdfFetch = _extRdflib["default"].rdfFetch;
var _default = GraphNode;
exports["default"] = _default;
//# sourceMappingURL=GraphNode.js.map